/*
    druckgott get RPM to Mach3 with:
 Orange RPM Sensor: http://www.hobbyking.com/hobbyking/store/uh_viewItem.asp?idProduct=66519
 
 Run Brushless Motor get PWM from Mach3 and change it to RC Sigal

//VERSION 1.5
 */


#include <Servo.h> 

Servo myservo;

#define DEBUG 0  

//CONFIG  
#define inPin  4     // pwm input from Mach3 to this Pin (Sub Pin 11 on Stepcraft)
#define runPin 5  // Connection for starting/stop Signal of the Spindle (Sub Pin 10 on Stepcraft)

#define escPin 6    // Connection for Brushless Motor to Signal pin on esc (normal orange/yellow cable on esc)

#define POLES 4 //number of poles for the Motor, otherwise you do not get the right rpm
#define esc_MOTOR_ON 1860 // Motor on ys
#define esc_MOTOR_OFF 1000  // Motor off ys

#define PWM_PERIOD  100000 // the number of microseconds for one PWM cycle which is configurate on Mach3
#define PWM_IDLE_STATE  HIGH   // set this to HIGH or LOW depending on how the mach3 outputs PWM
#define PWM_LOOP_ACC 1500	//speed up pro loop, first area (under 50%) darueber wird die Beschleunigung eingestellt
#define PWM_LOOP_ACC_2 6000	//speed up pro loop, second area (nearly 50%) darueber wird die Beschleunigung eingestellt
#define PWM_MIN_ACC 12000 //min output frequenz, bei der lauft die Spindel gerade so noch nicht, sofern das Signal aktiv an ist. Damit wird eine gewisse Totzeit ueberbrueckt bei der der Regler zwar schon stellt, der Motor selbst aber noch nicht anlauft. (am besten erst mal 0, dann ueber Debug den Wert Pruefen ab wann der Motor anfaengt zum laufen und die Zahl dann etwas tiefer halten (Mach3 in bevor acc:)

#define RPM_TO_MACH3 12 //Pin where Mach3 will read the rpm signal (Sub Pin 12 on Stepcraft)
#define INTERRUPT_PIN 0   // interrupt Pin: digital pin 2 http://arduino.cc/en/pmwiki.php?n=Reference/AttachInterrupt


int interruptCount = 0;
int trigger = 0; //trigger for low and high output of rpm signal to mach3
long old_pwm;
long old_time_pwm=PWM_MIN_ACC;
int check_running = NULL;

void setup()
{
  myservo.attach(escPin); //Initial the Servo
  pinMode(runPin, INPUT); //Initial Start/Stop Pin

  // setup for Output Pin MACH3
  pinMode(RPM_TO_MACH3, OUTPUT);
  // setup interrupt pin
  pinMode(INTERRUPT_PIN, INPUT);
  attachInterrupt(INTERRUPT_PIN, interruptFired, CHANGE);

  if(DEBUG){
    Serial.begin(9600);
  }
}

void loop()
{
  check_running=digitalRead(runPin);
  //check if the spindle should run
  if(check_running == LOW){
    myservo.writeMicroseconds(esc_MOTOR_OFF);
    old_time_pwm=PWM_MIN_ACC;
    if(DEBUG){
      Serial.println(" Motor off");
    }
  }

  if(check_running == HIGH){
    //get PWM signal from Mach3
    long  pwmIn = pulseIn( inPin,  PWM_IDLE_STATE );
    if(DEBUG){
      Serial.print("Mach3 in bevor acc: ");
      Serial.print(pwmIn);
    }
    //Pruefen ob die variable pwmIn einen Wert enthalt, sonst letzten Wert nehmen (damit immer ein Wert vorhanden ist)
    if(pwmIn==NULL){
      pwmIn=old_pwm;
    }
    else{
      old_pwm=pwmIn;
    }
    //Anlaufgeschwindigkeit einstellen unterscheiden ersten 50% und 2ten 50%
    if(((pwmIn>(old_time_pwm+PWM_LOOP_ACC)) or (pwmIn>(old_time_pwm+PWM_LOOP_ACC_2))) and (pwmIn <PWM_PERIOD)){
                if(old_time_pwm>(PWM_PERIOD/2)){
 		pwmIn=old_time_pwm+PWM_LOOP_ACC_2;
		old_time_pwm=old_time_pwm+PWM_LOOP_ACC_2;                 
                }else{
		pwmIn=old_time_pwm+PWM_LOOP_ACC;
		old_time_pwm=old_time_pwm+PWM_LOOP_ACC;
                }
	}
//Abbremsen immer gleich
    if((pwmIn<(old_time_pwm-PWM_LOOP_ACC)) and (pwmIn > 0)){
		pwmIn=old_time_pwm-PWM_LOOP_ACC;
		old_time_pwm=old_time_pwm-PWM_LOOP_ACC;
	}
    //hier wird der Bereich abgefangen, bei dem die Spindel noch nicht laeuft, damit das nicht zu begin ueberwunden werden muss bis die spindel zum laufen anfangt (esc gibt schon was aus, aber Motor dreht sich noch nicht)		
    if(old_time_pwm<PWM_MIN_ACC){
	old_time_pwm=PWM_MIN_ACC;
	}
    //Map PWM Signal from Mach3 to Servo Signal
    long escOut2 = map(pwmIn, 0, PWM_PERIOD, esc_MOTOR_OFF, esc_MOTOR_ON);
    //Move Servo/esc 
    myservo.writeMicroseconds(escOut2);
    if(DEBUG){
      Serial.print("Mach3 in after acc: ");
      Serial.print(pwmIn);
      Serial.print(" ESC out: ");
      Serial.println(escOut2);
    }
  }

  //delay(500);
}



//Fire Count interrupt
void interruptFired()
{

  //switch between low and high signal output for rpm input to mach3
  switch (trigger) {
  case 1:
    if(interruptCount == POLES*2){
      digitalWrite(RPM_TO_MACH3, HIGH);   //set HIGH
      //trigger 2 for next interrupt to switch to low
      trigger=2;			
      interruptCount=0;
    }
    break;
  case 2:
    if(interruptCount == POLES*2){
      digitalWrite(RPM_TO_MACH3, LOW);    //set LOW
      //trigger 1 for next interrupt to switch to high
      trigger=1;
      interruptCount=0;
    }
    break;
  default:
    digitalWrite(RPM_TO_MACH3, LOW);    
    trigger=1;       
    interruptCount=0;
  }

  interruptCount++;
}


